#ifndef _VWMTERM_H
#define _VWMTERM_H

#include <glib.h>
#include <gmodule.h>
#include <termios.h>

#include <sys/ioctl.h>

#include <vwm.h>
#include <viper.h>
#include <pseudo.h>

#ifdef _VIPER_WIDE
#include <ncursesw/curses.h>
#else
#include <curses.h>
#endif

/* define PTY_READ_MAX for the most liberal value.  fallback to the (POSIX).
   the most portable value. in the end, PTY_READ_MAX _should_ reflect the
   maximum number of bytes that the system is claiming to write to pipes
   atomically.  it's not necessary to read in atomic chunks but it seems wise
   to harmonize the pty read buffer size with this value.  */
#ifdef   PIPE_BUF
#define  PTY_READ_MAX         PIPE_BUF
#endif
#ifndef  PTY_READ_MAX
#ifdef   _POSIX_PIPE_BUF
#define  PTY_READ_MAX         _POSIX_PIPE_BUF   
#endif
#endif
#ifndef  PTY_READ_MAX
#define  PTY_READ_MAX         512
#endif

#define	PTY_STATE_FG			(1<<1)   /* tty is in the foreground   */
#define	PTY_STATE_ACS			(1<<2)   /* tty is now in alt-char-set mode  */
#define  PTY_STATE_CIVIS		(1<<3)   /* cursor is now invisible */
#define  PTY_STATE_CPARKED    (1<<4)   /* cursor is "parked" in bottom right  */
#define  PTY_STATE_CWRAPPED   (1<<5)   /* cursor just wrapped  */
#define  PTY_STATE_SCROLLOK   (1<<6)   /* scrolling is allowed.  this is
                                          not the same as scrollok().  instead
                                          this flag determines whether
                                          toggling scrollok() is permitted.   */

#define  VWMTERM_IS_SCROLLOK(vwmterm)  (vwmterm->new_state & PTY_STATE_SCROLLOK)

typedef struct
{
	WINDOW			*scrollback;
	WINDOW			*active_wnd;         /* TODO:  remove later. */
	pid_t				child_pid;
	gint				master_fd;
	gint				slave_fd;
	gchar				term_name[64];
	struct termios	term_desc;
   struct termios term_default;
	struct winsize	win_sz;
	gint				fg;
	gint				bg;
	gint				new_state;
	gint				old_state;
   gchar          *carryover;
   gint           carry_sz;
   
}
PTY_TERM;

G_MODULE_EXPORT gchar* g_module_check_init(GModule *module);

WINDOW*	ptyterm_init(gpointer anything);

/* pseudo thread	*/
gint     ptyterm_psthread(gpointer arg1,gpointer arg2);

/*	housekeeping helpers */

void     ptyterm_carryover(PTY_TERM *pty_term,gchar *pos_ptr,gint remainder);

/* data stream translation helpers  */
gint     ptyterm_xlate_discards(gchar *buffer);
gint     ptyterm_xlate_specials(PTY_TERM *pty_term,WINDOW *window,gchar *buffer);
gint     ptyterm_xlate_esc(PTY_TERM *pty_term,WINDOW *window,gchar *buffer);
void     ptyterm_xlate_sgr(PTY_TERM *pty_term,WINDOW *window,gint code);

#endif
