#include <unistd.h>

#include "vwmterm.h"
#include "watchdog.h"

void vwmterm_watchdog(PTY_TERM *pty_term)
{
	pid_t	            tcpid;

	/*	check to see if bash has regained control of the
		console.  this is important because some progs send a vt100 code which
		disables scrolling.  when we get control of the console back we need
		to enable scrolling just in case it was disabled	*/
	tcpid=tcgetpgrp(pty_term->master_fd);
	if(tcpid==-1) return;
  
	/*	store the value of the old state	*/
	pty_term->old_state=pty_term->new_state;

	if(tcpid!=pty_term->child_pid) pty_term->new_state &= (~PTY_STATE_FG);
	else pty_term->new_state |= PTY_STATE_FG;

   if(VWMTERM_IS_BG(pty_term) && VWMTERM_WAS_FG(pty_term))
   {
      /* TODO:  there are no known scenarios where fetching the terminal
         description serves a purpose.  it will be removed in the future
         if there are no compelling reasons to keep it.  */
      tcgetattr(pty_term->master_fd,&pty_term->term_desc);
   }

	/* these flags are tripped when the foreground process terminates (normally
		or because of a SIGKILL and then bash resumes control as the foreground
		process.  in the case of a KILL signal, ncurses can't issue the escape
		code to notify us to reset the terminal to scrollback mode so we do it
		manually based on this trigger	*/
	if(VWMTERM_IS_FG(pty_term) && VWMTERM_WAS_BG(pty_term))
	{
      /* update the terminal description. */
      tcgetattr(pty_term->master_fd,&pty_term->term_desc);

		viper_thread_enter();
      pty_term->active_wnd=pty_term->scrollback;
		pty_term->new_state &= (~PTY_STATE_CIVIS);
		wattrset(pty_term->active_wnd,A_NORMAL);
      scrollok(pty_term->active_wnd,TRUE);
      pty_term->new_state |= PTY_STATE_SCROLLOK;
		viper_thread_leave();
	}

	return;
}

