/*************************************************************************
 * All portions of code are copyright by their respective author/s.
 * Copyright (C) 2007      Bryan Christ <bryan.christ@hp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------*/

#include <string.h>
#include <inttypes.h>

#include <pseudo.h>

#include "vwm.h"
#include "vwm_menu.h"
#include "vwm_private.h"
#include "vwm_wndlist.h"
#include "vwm_scrsaver.h"
#include "vwm_syssaver.h"

/*
   pass back the struct sigaction so that the memory can be release if we
   need to replace the sighandler.  not use now, but maybe useful in the
   future.
*/
struct sigaction* vwm_sigset(int signum,sighandler_t handler)
{
   struct sigaction  *action;
   sigset_t          old_mask;

   if(handler==NULL) return NULL;

   action=(struct sigaction*)g_malloc0(sizeof(struct sigaction));

   /* retrieve current signal mask. */
   sigprocmask(0,NULL,&old_mask);
   action->sa_handler=handler;
   action->sa_mask=old_mask;
   sigaction(signum,(const struct sigaction*)action,NULL);

   return action;
}
   


gint vwm_hook_wm_start(WINDOW *window,gpointer arg)
{
	WINDOW	   *wallpaper;
   gchar       *help_msg;
   uintmax_t   msg_id;

	wallpaper=viper_screen_get_wallpaper();
	viper_thread_enter();	
	viper_screen_set_wallpaper(wallpaper,vwm_background,(gpointer)1);
	vwm_background(wallpaper,(gpointer)1);

   help_msg=g_strconcat("Press [Tab] to change window focus. ",
      "Use the keys [Up/Dn/Lt/Rt] to move windows. ",
      "Press [Ctrl+Q] to close a window.",NULL);

   msg_id=vwm_panel_message_find(VWM_MAIN_MENU_HELP);
   if(msg_id!=0) vwm_panel_message_del(msg_id);
   msg_id=vwm_panel_message_add(help_msg,-1);
   vwm_panel_message_promote(msg_id);
	viper_thread_leave();

   g_free(help_msg);  
   *(uintmax_t*)arg=msg_id;

	return 0;
}

gint vwm_hook_wm_stop(WINDOW *window,gpointer arg)
{
	WINDOW	   *wallpaper;
   uintmax_t   msg_id;
   
	msg_id=*(uintmax_t*)arg;

   viper_thread_enter();
	wallpaper=viper_screen_get_wallpaper();	
	viper_screen_set_wallpaper(wallpaper,vwm_background,(gpointer)0);
	vwm_background(wallpaper,(gpointer)0);

   vwm_panel_message_del(msg_id);
   vwm_panel_message_add(VWM_MAIN_MENU_HELP,-1);

   viper_thread_leave();
	
	return 0;
}

gint vwm_default_key_VWM_START(gint32 keystroke,gpointer anything)
{
	VWM			*vwm;
	VIPER_FUNC	hook_func=NULL;
	gpointer		hook_arg=NULL;

	vwm=vwm_get_instance();

	viper_event_run((WINDOW*)anything,"window-deactivate");
	hook_func=vwm->hook_enter;
	hook_arg=vwm->hook_enter_arg;
	if(hook_func!=NULL) hook_func((WINDOW*)anything,hook_arg);
	vwm_keyring_set(0);
	
	viper_screen_redraw(REDRAW_ALL);
	flash();
	
	return 0;
}

gint vwm_default_key_VWM_STOP(gint32 keystroke,gpointer anything)
{
	VWM			*vwm;
	VIPER_FUNC	hook_func=NULL;
	gpointer		hook_arg=NULL;

	vwm=vwm_get_instance();

	hook_func=vwm->hook_leave;
	hook_arg=vwm->hook_leave_arg;
	if(hook_func!=NULL) hook_func((WINDOW*)anything,hook_arg);
	viper_event_run((WINDOW*)anything,"window-activate");
	vwm_keyring_set(1);

	viper_screen_redraw(REDRAW_ALL);
	flash();
	
	return 0;
}
	
gint vwm_default_key_WINDOW_CLOSE(gint32 keystroke,gpointer anything)
{
	viper_event_run((WINDOW*)anything,"window-close");
	return 0;
}

gint vwm_default_key_WINDOW_CYCLE(gint32 keystroke,gpointer anything)
{
	viper_deck_cycle(VECTOR_BOTTOM_TO_TOP);
	return 0;
}

gint vwm_default_key_WINDOW_MOVE_UP(gint32 keystroke,gpointer anything)
{
	viper_mvwin_rel((WINDOW*)anything,0,-1);
	return 0;
}
	
gint vwm_default_key_WINDOW_MOVE_DOWN(gint32 keystroke,gpointer anything)
{
	viper_mvwin_rel((WINDOW*)anything,0,1);
	return 0;
}

gint vwm_default_key_WINDOW_MOVE_LEFT(gint32 keystroke,gpointer anything)
{
	viper_mvwin_rel((WINDOW*)anything,-1,0);
	return 0;
}

gint vwm_default_key_WINDOW_MOVE_RIGHT(gint32 keystroke,gpointer anything)
{
	viper_mvwin_rel((WINDOW*)anything,1,0);
	return 0;
}

gint vwm_background(WINDOW *window,gpointer arg)
{
	gint			idx;
   gshort      color=0;

#ifdef _VIPER_WIDE
	cchar_t			bg_char;
	wchar_t			ch[][2]={
						{0x0020,0x0000},
						{0x002e,0x0000}};
#else
	chtype			ch[]={' ','.'};
#endif

   /* TODO:  gcc warning...
      warning: cast from pointer to integer of different size
   */      	
	idx=(gint)arg;

	viper_thread_enter();

   viper_wresize_abs(window,WSIZE_FULLSCREEN,WSIZE_FULLSCREEN);

   if(idx==0) color=viper_color_pair(COLOR_WHITE,COLOR_BLUE);
   if(idx==1) color=viper_color_pair(COLOR_BLACK,COLOR_WHITE);
	
#ifdef _VIPER_WIDE
	setcchar(&bg_char,ch[idx],0,0,NULL);
	window_fill(window,&bg_char,color,A_NORMAL);
#else	
	window_fill(window,ch[idx],color,A_NORMAL);
#endif
	viper_thread_leave();
	
	return 0;
}

gint vwm_default_border_agent_focus(WINDOW *window,gpointer anything)
{
   VIPER_WND   *viper_wnd;
 	gint		   y,x;

   viper_wnd=(VIPER_WND*)anything;

   window_decorate(window,viper_wnd->title,TRUE);
   getmaxyx(viper_wnd->window,y,x);
	mvwprintw(viper_wnd->window,0,x-sizeof("[.OX]")+1,"[.OX]");

   if(!(viper_wnd->window_state & STATE_NORESIZE))
	mvwaddch(viper_wnd->window,y-1,x-1,'*');

   window_modify_border(viper_wnd->window,A_BOLD,
      viper_color_pair(COLOR_WHITE,COLOR_MAGENTA));

   return 0;
}

gint vwm_default_border_agent_unfocus(WINDOW *window,gpointer anything)
{
   VIPER_WND   *viper_wnd;
 	gint		   y,x;

   viper_wnd=(VIPER_WND*)anything;

   window_decorate(window,viper_wnd->title,TRUE);
   getmaxyx(viper_wnd->window,y,x);
	mvwprintw(viper_wnd->window,0,x-sizeof("[.OX]")+1,"[.OX]");

   if(!(viper_wnd->window_state & STATE_NORESIZE))
	mvwaddch(viper_wnd->window,y-1,x-1,'*');

   window_modify_border(viper_wnd->window,A_NORMAL,
		viper_color_pair(COLOR_BLACK,COLOR_CYAN));

   return 0;
}

void vwm_modules_preload(void)
{
	VWM      *vwm;
   WINDOW   *msgbox;
   gchar    *error_msg;
   gchar    *module_dirs[]={NULL,_VWM_SHARED_MODULES};
   gint     array_sz;
   gint     i;

	vwm=vwm_get_instance();

   array_sz=sizeof(module_dirs)/sizeof(module_dirs[0]);
   module_dirs[0]=VWM_MOD_DIR;
      
   for(i=0;i<array_sz;i++)
   {
      error_msg=vwm_modules_load(module_dirs[i]);
      if(error_msg!=NULL)
      {
         viper_thread_enter();
         msgbox=viper_msgbox_create(" Module Warning! ",0.5,0.5,0,0,error_msg,
            MSGBOX_ICON_WARN | MSGBOX_TYPE_OK);
         viper_window_show(msgbox);
         viper_thread_leave();
         g_free(error_msg);
      }
   }

 	/* these "fake" modules will appear on the menu in the same order whereby
		they were added below	*/
	vwm_module_add("VWM","Window List",vwm_fmod_wndlist,NULL,
		"vwm_fmod_winlist");
	vwm_module_add("VWM","Screensaver",vwm_fmod_scrsaver,NULL,
		"vwm_fmod_scrsaver");
	vwm_module_add("VWM","Exit",vwm_fmod_exit,NULL,
		"vwm_fmod_exit");

	/* special handling for default screensaver	*/
	vwm_module_add(VWM_SCREENSAVER,"SysSaver",vwm_fmod_syssaver,
      NULL,"vwm_fmod_sysinfo");
   vwm_scrsaver_start();
	
	return;
}

WINDOW* vwm_fmod_exit(gpointer anything)
{
	shmq_msg_put("ui",(gpointer)"shutdown vwm");
	
	return NULL;
}
