#include <time.h>

#ifdef _VIPER_WIDE
#include <ncursesw/curses.h>
#else
#include <curses.h>
#endif

#include <pseudo.h>

#include "vwm.h"
#include "vwm_psthread.h"
#include "vwm_scrsaver.h"

gint vwm_clock_driver(gpointer arg1,gpointer arg2)
{
   static GTimer  *timer=NULL;
   gfloat         elapsed;

   if(timer==NULL)
   {
      timer=g_timer_new();
      return PSTHREAD_CONTINUE;
   }

   elapsed=g_timer_elapsed(timer,NULL);
   if(elapsed<VWM_CLOCK_TICK) return PSTHREAD_CONTINUE;

   viper_thread_enter();
   viper_event_run(VIPER_EVENT_BROADCAST,"vwm-clock-tick");
   viper_thread_leave();

   g_timer_start(timer);
   return PSTHREAD_CONTINUE;
}

gint vwm_throttle_ui(gpointer arg1,gpointer arg2)
{
	static struct timespec	sleep_value;
	static glong				sleep_val;
	static guint				count_old=-1;
	guint							vcount;
	
	if(count_old==-1)
	{
		sleep_val=(glong)arg2;
		
		/* seconds */	
		sleep_value.tv_sec=(time_t)arg1;
		/* nanoseconds 0 - 999999999 */
		sleep_value.tv_nsec=sleep_val;

		nanosleep(&sleep_value,NULL);
		psthread_count("ui",NULL,&vcount);
		return PSTHREAD_CONTINUE;
	}

	psthread_count("ui",NULL,&vcount);
	if(vcount>count_old)
	{
		sleep_val=sleep_val>>1;
		if(sleep_val<0) sleep_val=0;
	}
	if(vcount<count_old)
	{
		sleep_val=sleep_val<<1;
		if(sleep_val>(glong)arg2) sleep_val=(glong)arg2;
	}
	
	sleep_value.tv_nsec=sleep_val;
	count_old=vcount;
	nanosleep(&sleep_value,NULL);
	
	return PSTHREAD_CONTINUE;
}

gint vwm_poll_input(gpointer arg1,gpointer arg2)
{
	gint32	keystroke;
 	
   viper_mouse_show();
	keystroke=viper_keystroke_get();

   if(keystroke==-1 || keystroke==KEY_RESIZE) return PSTHREAD_CONTINUE;
   vwm_scrsaver_reset();

	viper_thread_enter();
   /* vwm_hotkey_run will return -1 if the hotkey was processed, otherwise
      the keystroke value returned is unchanged.   */
	keystroke=vwm_hotkey_run(keystroke,(gpointer)TOPMOST_WINDOW);
	viper_thread_leave();
   
   viper_keystroke_dispatch(keystroke);
   return PSTHREAD_GROW;
}
